"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from __future__ import division
from math import ceil
from helpers import convertor
from distutils.version import LooseVersion
from constants import NOT_AVAILABLE, NOT_SUPPORTED


def get_info(cluster_obj, node_list):

    info = dict()
    node_list = node_list.replace(' ', '')
    node_list = convertor.text_to_list(node_list, ',')

    for node_address in node_list:
        node_obj = cluster_obj.get_node(node_address)
        info[node_address] = dict()
        if node_obj is not None and node_obj.get_status() == "on":
            info[node_address]['node_status'] = 'on'
            if node_obj.is_valid() is True:
                info[node_address]['same_cluster'] = "True"
                info[node_address]['memory'] = node_obj.get_memory_stats()
                info[node_address]['disk'] = node_obj.get_disk_stats()
                info[node_address]['build'] = node_obj.get_build()
                stats = node_obj.get_subset_of_statistics([
                    "objects", "client_connections", "cluster_size",
                    "migrate_progress_send", "migrate_progress_recv",
                    "cluster_integrity", "err_out_of_space", "queue",
                    "write_master", "write_prole", "system_swapping",
                    "err_write_fail_prole_unknown", "stat_duplicate_operation",
                    "uptime", "system_free_mem_pct", "batch_errors",
                    "storage_defrag_records", "stat_write_errs",
                    "free-pct-memory", "free-pct-disk", "node_status",
                    'cluster_visibility'
                    ])
                info[node_address].update(stats)

                migrate_incoming_pct, migrate_outgoing_pct = \
                    calculate_migrate_stats(node_obj)
                info[node_address]['migrate_incoming_pct'] = \
                    migrate_incoming_pct
                info[node_address]['migrate_outgoing_pct'] = \
                    migrate_outgoing_pct

                configurations = node_obj.get_subset_of_configurations([
                    "proto-fd-max", "client-fd-max", "nsup-threads",
                    "scan-priority", "storage-benchmarks",
                    "replication-fire-and-forget"
                    ])
                info[node_address].update(configurations)
            elif node_obj.is_valid() is False:
                info[node_address]['same_cluster'] = "False"
                cluster_obj.remove_node(node_address)
        else:
            info[node_address]['node_status'] = 'off'

    return info


def get_stats(cluster_obj, node_address, stats_type="all"):

    #info = dict(address=node_address)
    info = dict()
    node_obj = cluster_obj.get_node(node_address)

    if node_obj is None or not node_obj.is_valid():
        info['node_status'] = "off"
    else:
        stats = node_obj.get_statistics() if stats_type == "all" else node_obj.get_modified_statistics()
        info.update(stats)
        configurations = node_obj.get_configurations() if stats_type == "all" else node_obj.get_modified_configurations()
        if configurations is None:
            configurations = node_obj.get_updated_configurations()
        if configurations is not None:
            info.update(configurations)

    return info


def calculate_migrate_stats(node_obj):
    migrate_incoming_pct = NOT_SUPPORTED
    migrate_outgoing_pct = NOT_SUPPORTED

    build = node_obj.get_build()
    if build and LooseVersion(build) >= LooseVersion('3.7.0'):

        migrate_incoming_pct = NOT_AVAILABLE
        migrate_outgoing_pct = NOT_AVAILABLE

        migrate_tx_partitions_initial = 0
        migrate_tx_partitions_remaining = 0
        migrate_rx_partitions_initial = 0
        migrate_rx_partitions_remaining = 0

        # Iterate namespaces
        for ns, ns_obj in node_obj.get_namespaces().items():
            migrate_stats = ns_obj.get_subset_of_statistics(
                ['migrate-rx-partitions-initial',
                 'migrate-rx-partitions-remaining',
                 'migrate-tx-partitions-initial',
                 'migrate-tx-partitions-remaining'])

            for key, value in migrate_stats.items():
                try:
                    migrate_stats[key] = int(value)
                except ValueError:
                    migrate_stats[key] = 0

            migrate_rx_partitions_initial += migrate_stats[
                'migrate-rx-partitions-initial']
            migrate_rx_partitions_remaining += migrate_stats[
                'migrate-rx-partitions-remaining']
            migrate_tx_partitions_initial += migrate_stats[
                'migrate-tx-partitions-initial']
            migrate_tx_partitions_remaining += migrate_stats[
                'migrate-tx-partitions-remaining']

        # Calculate Migrate Incoming %
        if migrate_rx_partitions_initial != 0:
            try:
                migrate_incoming_pct = (
                    1 - (migrate_rx_partitions_remaining /
                         migrate_rx_partitions_initial)) * 100
                migrate_incoming_pct = int(ceil(migrate_incoming_pct))
            except Exception:
                migrate_incoming_pct = NOT_AVAILABLE

        # Calculate Migrate Outgoing %
        if migrate_tx_partitions_initial != 0:
            try:
                migrate_outgoing_pct = (
                    1 - (migrate_tx_partitions_remaining /
                         migrate_tx_partitions_initial)) * 100
                migrate_outgoing_pct = int(ceil(migrate_outgoing_pct))
            except Exception:
                migrate_outgoing_pct = NOT_AVAILABLE

    return migrate_incoming_pct, migrate_outgoing_pct
